/* Include file for high-level encryption routines.
 *
 * PTlink Services is (C) CopyRight PTlink Coders Team 1999 
 * http://www.ptlink.net/Coders - coders@PTlink.net
 * This program is distributed under GNU Public License
 * Please read the file COPYING for copyright information.
 *
 * These services are based on Andy Church Services
 */

#include "services.h"
#include "encrypt.h"
#include <string.h>

/* Encrypt `src' of length `len' and store the result in `dest'.  If the
 * resulting string would be longer than `size', return -1 and leave `dest'
 * unchanged; else return 0.
 */
int encrypt_jp2(const char *src, int len, char *dest, int size)
{
/* Simple XOR encryption algorithm - Joao Pinto 1999 */
    char buf[BUFSIZE];
    char sum=0;
    int i;
    strcpy(buf,src);
    for(i=0;i<len;++i) sum+=src[i];    
    buf[--i]|=sum; 
    i--;
    for(;i>=0;--i) { 
    	buf[i]^=src[i+1];
	if(buf[i]=='\0') buf[i]='!'; /* make sure passwords are not truncated*/
    }
    strcpy(dest,buf);
    return 0;
}

/* Simple two ways XOR encryption algorithm - Joao Pinto 1999 */
int encrypt_jp_rev(const char *src, int len, char *dest, int size)
{
    char buf[BUFSIZE];
    char sum=0;
    int i;
    strcpy(buf,src);
    for(i=0;i<len;++i) sum+=src[i];    
    buf[--i]|=sum; 
    i--;
    for(;i>=0;--i) { 
    	buf[i]^=sum;
	if(buf[i]=='\0') buf[i]='!'; /* make sure passwords are not truncated */
    }
    strcpy(dest,buf);
    return 0;
}


int encrypt(const char *src, int len, char *dest, int size)
{
    switch(EncryptMethod) {
    case 0:
	strncpy(dest,src,len);
	dest[len]='\0';		/* bastard merging passwords:P */
	return 1;
	break;
    case 1:
	return encrypt_jp2(src, len, dest, size);
	break;
    }
    return -1;
}

/* Shortcut for encrypting a null-terminated string in place. */
int encrypt_in_place(char *buf, int size)
{
    switch(EncryptMethod) {
    case 0:
	return 1;
	break;
    case 1:
	return encrypt_jp2(buf, strlen(buf), buf, size);
	break;
    }
    return -1;
}


/* Compare a plaintext string against an encrypted password.  Return 1 if
 * they match, 0 if not, and -1 if something went wrong. */

int check_password(const char *plaintext, const char *password, int crypt_method) 
{
    char buf[BUFSIZE];
    switch(crypt_method) {
    case 0:
	return strcmp(plaintext, password) ? 0 : 1;
	break;
    case 1:	
	if (encrypt_jp2(plaintext, strlen(plaintext), buf, sizeof(buf)) < 0)
	    return -1;
	return	memcmp(buf,password,strlen(password)) ? 0 : 1;
	break;
    }
    log("Unknon encryption method: %i",crypt_method);
    return -1;
}

/*************************************************************************/

